@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -30,17 +28,19 @@
 
 // Libkern includes
 #include <libkern/OSByteOrder.h>
+#include <libkern/OSAtomic.h>
 
 // General IOKit includes
 #include <IOKit/IOWorkLoop.h>
 #include <IOKit/IOCommandGate.h>
 
 // SCSI Architecture Model Family includes
+#include "IOSCSIProtocolServices.h"
 #include <IOKit/scsi/IOSCSITargetDevice.h>
-#include <IOKit/scsi/IOSCSIProtocolServices.h>
 
 #include "SCSITaskDefinition.h"
 
+#define fSemaphore	fIOSCSIProtocolServicesReserved->fSemaphore
 
 //
 //	Macros
@@ -95,6 +95,18 @@ static IOPMPowerState sPowerStates[kSCSIProtocolLayerNumDefaultStates] =
 	{ 1, (IOPMDeviceUsable | IOPMMaxPerformance), IOPMPowerOn, IOPMPowerOn, 0, 0, 0, 0, 0, 0, 0, 0 }
 };
 
+enum
+{
+	kSCSITaskQueueBusyBit		= 0,
+	kSCSITaskQueueCompletionBit	= 1
+};
+
+enum
+{
+	kSCSITaskQueueBusyMask			= ( 1 << kSCSITaskQueueBusyBit ),
+	kSCSITaskQueueCompletionMask	= ( 1 << kSCSITaskQueueCompletionBit )
+};
+
 
 #if 0
 #pragma mark -
@@ -103,24 +115,6 @@ static IOPMPowerState sPowerStates[kSCSIProtocolLayerNumDefaultStates] =
 #endif
 
 
-//
-//	 init - Initialization method.						 			   [PUBLIC]
-//
-
-bool
-IOSCSIProtocolServices::init ( OSDictionary * propTable )
-{
-	
-	if ( super::init ( propTable ) == false )
-	{
-		return false;
-	}
-	
-	return true;
-	
-}
-
-
 //
 //	 start - Called by IOKit to start our services.		  		   [PUBLIC]
 //
@@ -129,12 +123,13 @@ bool
 IOSCSIProtocolServices::start ( IOService * provider )
 {
 	
-	OSDictionary *  dict = NULL;	
+	OSDictionary *  dict 	= NULL;	
+	bool			result	= false;
 	
-	if ( !super::start ( provider ) )
-	{
-		return false;
-	}
+	result = super::start ( provider );
+	require ( result, ErrorExit );
+	
+	result = false;
 	
 	// Setup to allow service requests
 	fAllowServiceRequests = true;
@@ -142,9 +137,15 @@ IOSCSIProtocolServices::start ( IOService * provider )
 	// Initialize the head pointer for the SCSI Task Queue.
 	fSCSITaskQueueHead = NULL;
 	
+	fIOSCSIProtocolServicesReserved = IONew ( IOSCSIProtocolServicesExpansionData, 1 );
+	require_nonzero ( fIOSCSIProtocolServicesReserved, ErrorExit );
+	
+	// Zero the reserved data section.
+	bzero ( fIOSCSIProtocolServicesReserved, sizeof ( IOSCSIProtocolServicesExpansionData ) );
+	
 	// Allocate the mutex for accessing the SCSI Task Queue.
 	fQueueLock = IOSimpleLockAlloc ( );
-	require ( fQueueLock, PanicNow );
+	require_nonzero ( fQueueLock, FreeReserved );
 	
 	// If the provider has a Protocol Characteristics dictionary, copy
 	// it to the Protocol Services object.
@@ -152,23 +153,31 @@ IOSCSIProtocolServices::start ( IOService * provider )
 	if ( dict != NULL )
 	{
 		
-		OSDictionary *	protocolDict = NULL;
+		OSDictionary *	protocolDict 	= NULL;
 		
 		protocolDict = OSDictionary::withDictionary ( dict );
 		setProperty ( kIOPropertyProtocolCharacteristicsKey, protocolDict );
 		protocolDict->release ( );
 		
-	}
+	}	
 	
-	return true;
+	result = true;
+	
+	return result;
 	
 	
-PanicNow:
+FreeReserved:
 	
 	
-	IOPanic ( "SCSIProtcolServices: Could not allocate simple lock for task queue\n" );
+	require_nonzero_quiet ( fIOSCSIProtocolServicesReserved, ErrorExit );
+	IODelete ( fIOSCSIProtocolServicesReserved, IOSCSIProtocolServicesExpansionData, 1 );
+	fIOSCSIProtocolServicesReserved = NULL;
 	
-	return false;
+	
+ErrorExit:
+	
+	
+	return result;
 	
 }
 
@@ -190,6 +199,14 @@ IOSCSIProtocolServices::free ( void )
 		
 	}
 	
+	if ( fIOSCSIProtocolServicesReserved != NULL )
+	{
+		
+		IODelete ( fIOSCSIProtocolServicesReserved, IOSCSIProtocolServicesExpansionData, 1 );
+		fIOSCSIProtocolServicesReserved = NULL;
+		
+	}
+	
 	super::free ( );
 	
 }
@@ -984,27 +1001,56 @@ void
 IOSCSIProtocolServices::SendSCSITasksFromQueue ( void )
 {
 	
-	bool			cmdAccepted = false;
-	SCSITask *		nextVictim;
-	
-	do 
+	// Is there anything in the queue?
+	while ( fSCSITaskQueueHead != NULL ) 
 	{
 		
-		SCSIServiceResponse 	serviceResponse;
-		SCSITaskStatus			taskStatus;
+		bool	qDrained = false;
 		
-		// get the next command from the request queue
-		nextVictim = RetrieveNextSCSITaskFromQueue ( );
-		if ( nextVictim != NULL )
+		// Do we need to drive the queue?
+		if ( OSBitOrAtomic ( kSCSITaskQueueBusyMask, &fSemaphore ) & kSCSITaskQueueBusyMask )
 		{
 			
+			// Someone else is driving the queue, break out and let them do it.
+			break;
+			
+		}
+		
+		// Send as many commands as there are in the queue, or upto the point
+		// the transport driver tells us it can't handle any more.
+		
+		while ( true )
+		{
+			
+			SCSIServiceResponse 	serviceResponse;
+			SCSITaskStatus			taskStatus;
+			SCSITask *				nextVictim	= NULL;
+			bool					cmdAccepted = false;
+			
+			// We're sending a command down, so clear the completion bit so
+			// we know if a completion occurred while we were sending a command.
+			OSBitAndAtomic ( ~kSCSITaskQueueCompletionMask, &fSemaphore );
+			
+			// Get the next command from the request queue
+			nextVictim = RetrieveNextSCSITaskFromQueue ( );
+			if ( nextVictim == NULL )
+			{
+				
+				// No command in the queue, break out of the loop because
+				// we drained the queue.
+				qDrained = true;
+				break;
+				
+			}
+			
 			cmdAccepted = SendSCSICommand ( nextVictim, &serviceResponse, &taskStatus );
 			if ( cmdAccepted == false )
 			{
 				
 				// The subclass can not process the command at this time,
-				// add it to the queue and try again at CommandComplete time.
+				// add it to the queue and try again later.
 				AddSCSITaskToHeadOfQueue ( nextVictim );
+				break;
 				
 			}
 			
@@ -1020,8 +1066,23 @@ IOSCSIProtocolServices::SendSCSITasksFromQueue ( void )
 			
 		}
 		
-	} while ( ( cmdAccepted == true ) && ( nextVictim != NULL ) );
-
+		// We aren't driving the queue any more. Mark that we aren't doing so.
+		OSBitAndAtomic ( ~kSCSITaskQueueBusyMask, &fSemaphore );
+		
+		// We broke out of the while loop above either because there are no more
+		// commands to process, or the transport driver cannot handle any more at
+		// this time. Check if any completions occurred since we released the queue
+		// driving privilege. If so, we attempt to keep processing the queue.
+		if ( ( qDrained == false ) && ( ( fSemaphore & kSCSITaskQueueCompletionMask ) == 0 ) )
+		{
+			// No completions, break out.
+			break;
+		}
+		
+		// A completion did occur. Start over...
+		
+	}
+	
 }
 
 
@@ -1202,6 +1263,8 @@ IOSCSIProtocolServices::CommandCompleted ( 	SCSITaskIdentifier 	request,
 		
 	}
 	
+	OSBitOrAtomic ( kSCSITaskQueueCompletionMask, &fSemaphore );
+	
 	ProcessCompletedTask ( request, serviceResponse, taskStatus );
 	
 	SendSCSITasksFromQueue ( );
