@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -709,8 +707,6 @@ SCSITaskUserClient::ReleaseTask ( SInt32 taskReference )
 	SCSITaskRefCon *		refCon	= NULL;
 	
 	STATUS_LOG ( ( "SCSITaskUserClient::ReleaseTask\n" ) );
-
-	require_action ( isInactive ( ) == false, GENERAL_ERR, status = kIOReturnNoDevice );
 	
 	require ( ( taskReference >= 0 ) && ( taskReference < kMaxSCSITaskArraySize ),
 			  GENERAL_ERR );
@@ -868,7 +864,7 @@ SCSITaskUserClient::ExecuteTask ( SCSITaskData * args, UInt32 argSize )
 			
 			// Make sure to complete any data buffers from client
 			status = CompleteBuffers ( buffer );
-		
+			
 		}
 		
 		release ( );
@@ -2565,7 +2561,11 @@ SCSITaskUserClient::didTerminate ( IOService * 		provider,
 								   bool *			defer )
 {
 	
-	HandleTerminate ( provider );	
+	if ( fOutstandingCommands == 0 )
+	{
+		HandleTerminate ( provider );
+	}
+	
 	return true;
 	
 }
@@ -2593,7 +2593,7 @@ SCSITaskUserClient::HandleTerminate ( IOService * provider )
 		provider->close ( this, kIOSCSITaskUserClientAccessMask );
 		
 	}
-		
+	
 	// Decouple us from the IORegistry.
 	detach ( provider );
 	fProvider = NULL;
@@ -2759,7 +2759,7 @@ SCSITaskUserClient::TaskCallback ( SCSITask * task, SCSITaskRefCon * refCon )
 		
 		OSAsyncReference	asyncRef;
 		
-		asyncRef = refCon->asyncReference;
+		bcopy ( refCon->asyncReference, asyncRef, sizeof ( OSAsyncReference ) );
 		
 		STATUS_LOG ( ( "asyncRef[0] = %d\n", asyncRef[0] ) );
 		
@@ -2780,6 +2780,9 @@ SCSITaskUserClient::TaskCallback ( SCSITask * task, SCSITaskRefCon * refCon )
 		
 	}
 	
+	if ( isInactive ( ) && ( fOutstandingCommands == 0 ) )
+		HandleTerminate ( fProvider );
+	
 }
 
 
@@ -2796,12 +2799,14 @@ SCSITaskUserClient::SetupTask ( SCSITask ** task )
 	
 	check ( task );
 	
-	newTask = new SCSITask;
+	newTask = OSTypeAlloc ( SCSITask );
 	require_nonzero ( newTask, TASK_CREATE_ERR );
 	require_action ( newTask->ResetForNewTask ( ),
 					 TASK_CREATE_ERR,
 					 newTask->release ( ) );
 	
+	newTask->SetTaskOwner ( this );
+	
 	*task 	= newTask;
 	status	= kIOReturnSuccess;
 	
@@ -2960,20 +2965,22 @@ void
 SCSITaskUserClient::sTaskCallback ( SCSITaskIdentifier completedTask )
 {
 	
-	SCSITaskRefCon *		refCon	= NULL;
 	SCSITask *				task	= NULL;
+	SCSITaskRefCon *		refCon	= NULL;
+	SCSITaskUserClient *	uc		= NULL;
 	
 	STATUS_LOG ( ( "SCSITaskUserClient::sTaskCallback called.\n") );
 	
 	task = OSDynamicCast ( SCSITask, completedTask );
-	require_nonzero_string ( task, GENERAL_ERR,
-							 "SCSITaskUserClient::sTaskCallback task == NULL." );
+	require_nonzero ( task, GENERAL_ERR );
 	
 	refCon = ( SCSITaskRefCon * ) task->GetApplicationLayerReference ( );
-	require_nonzero_string ( task, GENERAL_ERR,
-							 "SCSITaskUserClient::sTaskCallback refCon == NULL." );
+	require_nonzero ( refCon, GENERAL_ERR );
 	
-	refCon->self->TaskCallback ( task, refCon );
+	uc = refCon->self;
+	require_nonzero ( uc, GENERAL_ERR );
+	
+	uc->TaskCallback ( task, refCon );
 	
 	
 GENERAL_ERR:
